/*
Copyright (c) 2005, Apple Computer, Inc.  All rights reserved.
NOTE:  Use of this source code is subject to the terms of the Software
License Agreement for Mac OS X, which accompanies the code.  Your use
of this source code signifies your agreement to such license terms and
conditions.  Except as expressly granted in the Software License Agreement
for Mac OS X, no other copyright, patent, or other intellectual property
license or right is granted, either expressly or by implication, by Apple.
*/


var ChineseS = ['en'];
var ChineseT = ['en'];
var Dutch = ['en','fr'];
var English = [
    'zh-Hans',
    'zh-Hant',
    'nl',
    'fr',
    'de',
    'el',
    'it',
    'ja',
    'ko',
    'pt',
    'ru',
    'es'
];
var French = ['nl','en','de','el','it','pt','es'];
var German = ['en','fr'];
var Greek = ['en','fr'];
var Italian = ['en','fr'];
var Japanese = ['en'];
var Korean = ['en'];
var Portuguese = ['en','fr'];
var Russian = ['en'];
var Spanish = ['en', 'fr'];

var languageData = [
        {name:'Chinese (Simplified)', code:"zh", iso:"zh-Hans", array:ChineseS},
        {name:'Chinese (Traditional)', code:"zt", iso:"zh-Hant", array:ChineseT},
        {name:'Dutch', code:"nl", iso:"nl", array:Dutch},
        {name:'English', code:"en", iso:"en", array:English},
        {name:'French', code:"fr", iso:"fr", array:French},
        {name:'German', code:"de", iso:"de", array:German},
        {name:'Greek', code:"el", iso:"el", array:Greek},
        {name:'Italian', code:"it", iso:"it", array:Italian},
        {name:'Japanese', code:"ja", iso:"ja", array:Japanese},
        {name:'Korean', code:"ko", iso:"ko", array:Korean},
        {name:'Portuguese', code:"pt", iso:"pt", array:Portuguese},
        {name:'Russian', code:"ru", iso:"ru", array:Russian},
        {name:'Spanish', code:"es", iso:"es", array:Spanish}
];

var resizeAnimation = {startTime:0, duration:250, timer:null, list:null, onfinished:null};

var defaultFromLanguage = 3;   // use English as the default from language
var defaultToLanguage = 4;     // use French as the default to language

// gAllowTextToGrow is used to determine when we will let the user add 
//	to the "from" text
// Once either the "to" or the "from" text would require more than six
//	lines, we stop accepting additions to the "from" text
var gAllowTextToGrow = true;

var kLineWidth = 291.0;
var kMaxNumLines = 6;
var kMinNumLines = 3;
var kDefaultLineHeight = 18.0;
var kMinTextBoxHeight = kMinNumLines * kDefaultLineHeight;
var kDefaultTextSize = 15.0;

var kAbsoluteMaximumWidgetHeight = 750;

var kAnimationFrameMilliseconds = 13;

var fromLanguageFont = "Helvetica Neue";
var fromLanguageSize = kDefaultTextSize;
var fromLanguageLineHeight = kDefaultLineHeight;
var toLanguageFont = "Helvetica Neue";
var toLanguageSize = kDefaultTextSize;
var toLanguageLineHeight = kDefaultLineHeight;

var requestID = 0;
var receiveID = 0;

var timerInterval = null;

var lastEncodedBlock = '';

// support for parser
var gLastTranslationXMLRequest = null;
var apple_parser = new AppleParser ("http://www.apple.com/widgets/translationParser.js");
apple_parser.performXMLRequest = performXMLRequest;

function doBeep()
{
	if( !gAllowTextToGrow && TranslationEncoder != null ) TranslationEncoder.beep();
}

function fontForLanguage(language)
{
	var returnValue = "Helvetica Neue";
	if ( language == "el" || language == "ru" )
		returnValue = "Helvetica";
	else if ( language == "ja" )
		returnValue = "Hiragino Maru Gothic Pro";
	else if ( language == "ko" )
		returnValue = "AppleGothic";
	else if ( language == "zh-Hans" )
		returnValue = "Hei";
	else if ( language == "zh-Hant" )
		returnValue = "LiHei Pro";
	return returnValue;
}


function lineHeightForFont(fontName)
{
	if ( fontName == "Helvetica Neue" )
		return 18;
	else if ( fontName == "Helvetica" )
		return 19;
	else if ( fontName == "Hiragino Maru Gothic Pro" )
		return 23;
	else if ( fontName == "AppleGothic" )
		return 18;
	else if ( fontName == "Hei" )
		return 16;
	else if ( fontName == "LiHei Pro" )
		return 16;
	else return 18;		/* We should never get here */
}

function createKey (key)
{
	return widget.identifier + "-" + key
}

function setPreferenceForKey(pref, key)
{
	widget.setPreferenceForKey(pref,key);
}

function getPreferenceForKey(key)
{
	pref = widget.preferenceForKey(key);	
	return pref;
}

function CBGImage( containerID, topLID, topID, topRID, leftID, centerID, rightID, bottomLID, bottomID, bottomRID )
{
	this.fContainer = createElementRecord( containerID );//cont;
	// top
	this.fTopL = createElementRecord( topLID ); //topL
	this.fTop = createElementRecord( topID ); //top;
	this.fTopR = createElementRecord( topRID ); //topR;
	// center
	this.fLeft = createElementRecord( leftID ); //left;
	this.fCenter = createElementRecord( centerID ); //center;
	this.fRight = createElementRecord( rightID ); //right;
	//bottom
	this.fBottomL = createElementRecord( bottomLID ); //bottomL;
	this.fBottom = createElementRecord( bottomID ); //bottom;
	this.fBottomR = createElementRecord( bottomRID ); //bottomR;
	
	this.getElement = function( member ) { return document.getElementById( member.id ) };
	this.growElement = _growElement;
	this.resizeBy = _resizeBy;
	this.resizeTo = _resizeTo;
	this.offsetPos = _offsetPos;
	this.setOrigin = _setOrigin;
	
	this.getHeight = function() { return this.fContainer.height };
	this.getOrigin = function() { return { left:this.fContainer.left, top:this.fContainer.top } };
	
	function _growElement( member, dhoriz, dvert )
	{
		var element = this.getElement( member );
		
		element.style.height = member.height + dvert;
		element.style.width = member.width + dhoriz;
	}
	
	function _resizeTo( width, height )
	{
		var currentWidth = this.fContainer.width;
		var currentHeight = this.fContainer.height;
		
		// less than 1 width will be ignored
		var deltaWidth = 0;
		if( width > 0 )
			currentWidth < width ? width - currentWidth : currentWidth - width;
			
		// less than 1 height will be ignored
		var deltaHeight = 0;
		if( height > 0 )
			deltaHeight = height - currentHeight;
		
		this.resizeBy( deltaWidth, deltaHeight );
		
		return { d_width:deltaWidth, d_height:deltaHeight };
	}
	
	function _resizeBy( width, height )
	{
		if( this.fLeft != null )
		{
			var left = this.getElement(this.fLeft);
			this.fLeft.height = this.fLeft.height + height;
			left.style.height = this.fLeft.height;
		}
		
		if( this.fRight != null )
		{
			var right = this.getElement(this.fRight);
			this.fRight.height = this.fRight.height + height;
			right.style.height = this.fRight.height;
		}
		
		if( this.fCenter != null )
		{
			var center = this.getElement(this.fCenter);
			this.fCenter.height = this.fCenter.height + height;
			center.style.height = this.fCenter.height;
		}
		
		var container = this.getElement( this.fContainer );
		
		this.fContainer.width = this.fContainer.width + width;
		container.style.width = this.fContainer.width;
		
		this.fContainer.height = this.fContainer.height + height;
		container.style.height = this.fContainer.height;
		
		var flipButton = document.getElementById('flip');
		flipButton.style.top = this.fContainer.top+this.fContainer.height - 17;
		
		var flipRollie = document.getElementById('flip_rollie');
		flipRollie.style.top = this.fContainer.top+this.fContainer.height - 17;
	}
	
	function _offsetPos( dhoriz, dvert )
	{
		var container = this.getElement( this.fContainer );
		this.fContainer.left = this.fContainer.left + dhoriz;
		container.style.left = this.fContainer.left;
		this.fContainer.top = this.fContainer.top + dvert
		container.style.top = this.fContainer.top;
	}
	
	function _setOrigin( left, top )
	{
		var container = this.getElement( this.fContainer );
		
		if( left >= 0 ) // you can pass -1 if you only want to set one param
		{
			this.fContainer.left = left;
			container.style.left = this.fContainer.left;
		}
		if( top >= 0 )
		{
			this.fContainer.top = top;
			container.style.top = this.fContainer.top;
		}
	}
}

function CTextArea( ident, textBGimg, label_id, lang_id, popup_id, popupBGimg_id, hiderID)
{
	// id's of the elements associated with an instance of this class
	
	// main element
	//this.f_id = ident;
	
	this.fMainElement = createElementRecord( ident );
	this.fLastTextLen = 0;
	this.fLastLineCount = 0;
	
	this.f_TextBGimg = textBGimg;
	
	// elements which take a position relative to the main element
	this.flabel_id = label_id;	// label ("From" or "To")
	this.flang_id = lang_id;	// static language menu value
	this.fPopup_id = popup_id;	// popup menu
	this.fPopupBGimg_id = popupBGimg_id; // static popup menu bg image
	
	this.fHider = hiderID;
	
	// element accessors
	this.getElement = function( elem_id ) { return document.getElementById( elem_id) };
	
	// function member assignments for functions defined below
	this.setRect = _setRect;
	this.offsetPos = _offsetPos;
	this.offsetTo = _offsetTo;
	this.setRows = _setRows;
	this.resizeTo = _resizeTo;
	this.resizeBy = _resizeBy;
	this.getHeight = _getHeight;
	this.getOrigin = _getOrigin;
	this.getLeft = _getLeft;
	this.updateRelativeItems = _updateRelativeItems;
	this.focus = _focus;
	
	function _focus( focusOn )
	{
		// In Tiger, <textarea> has a grey lined border around it 
		// which cannot be eliminated by setting CSS properties.
		// We introduce one new div for each text area which will be used
		// as a workaround to "hide" the grey borders.
		
		// The new divs have transparent background, but white borders.  
		// We adjust the z-index of the textareas as appropriate.  
		// We want the "hider" divs to go behind the text area they are 
		// covering, if that area gets focused, and to come forward / above \
		// when the text area loses focus.
		
		var textField = this.getElement( this.fMainElement.id );
		if( focusOn )
		{
			textField.style.zIndex=21;
			textField.focus();
		}
		else
		{
			textField.style.zIndex=15;
			textField.blur();
		}
	}
	
	function _getHeight()
	{
	   return this.fMainElement.height;
	}
	
	function _getOrigin()
	{
	   return this.fMainElement.top;
	}
	
	function _getLeft()
	{
	   return this.fMainElement.left;
    }
	
	function _setRect(l,t,h,w)
	{
		this.fMainElement.left = l;
		this.fMainElement.top = t;
		this.fMainElement.height = h;
		this.fMainElement.width = w;
	}
	
	function _offsetTo( left, top )
	{
	   var deltaHoriz = left - this.fMainElement.left;
	   var deltaVert = top - this.fMainElement.top;
	   
	   this.offsetPos( deltaHoriz, deltaVert );
	}
	
	function _offsetPos( horiz, vert )
	{
		this.fMainElement.left = this.fMainElement.left + horiz;
		this.fMainElement.top = this.fMainElement.top + vert;
		
		var mainElement = this.getElement( this.fMainElement.id );
		mainElement.style.left = this.fMainElement.left;
		mainElement.style.top = this.fMainElement.top;
		
		var hider = this.getElement( this.fHider );
		hider.style.left = this.fMainElement.left;
		hider.style.top = this.fMainElement.top;
		
		this.updateRelativeItems( horiz, vert );
	}
	
	function _setRows( rowCount )
	{
		this.getElement( this.fMainElement.id ).rows = rowCount;
	}

	function _resizeTo( width, height )
	{
		var mainElement = this.getElement( this.fMainElement.id );
		
		var currentWidth = this.fMainElement.width;
		var currentHeight = this.fMainElement.height;
		
		// less than 1 width will be ignored
		var deltaWidth = 0;
		if( width > 0 )
			currentWidth < width ? width - currentWidth : currentWidth - width;
			
		// less than 1 height will be ignored
		var deltaHeight = 0;
		if( height > 0 )
			deltaHeight = height - currentHeight;
		
		this.resizeBy( deltaWidth, deltaHeight );
		
		return { d_width:deltaWidth, d_height:deltaHeight };
	}
	
	function _resizeBy( deltaWidth, deltaHeight )
	{
		var mainElement = this.getElement( this.fMainElement.id );
		
		this.fMainElement.height = this.fMainElement.height + deltaHeight;
		this.fMainElement.width = this.fMainElement.width + deltaWidth;
		
		mainElement.style.height = this.fMainElement.height;
		mainElement.style.width = this.fMainElement.width;
		
		var hider = this.getElement( this.fHider );
		hider.style.height = this.fMainElement.height-2;
		hider.style.width = this.fMainElement.width-2;
		
		this.f_TextBGimg.resizeBy( deltaWidth /*width*/, deltaHeight/*height*/);
		
		var unavailableLbl = document.getElementById('unavailable-label');
		unavailableLbl.style.top = parseInt(unavailableLbl.style.top) + deltaHeight;
	}	
	
	function _updateRelativeItems( d_horiz, d_vert)
	{
		var constPopupOffset = 37;
		
		var label = this.getElement( this.flabel_id );
		label.style.top = this.fMainElement.top-(constPopupOffset-3);
		
		var lang = this.getElement( this.flang_id);
		lang.style.top = this.fMainElement.top-(constPopupOffset-3);
		
		var popupMenu = this.getElement( this.fPopup_id );
		popupMenu.style.top = this.fMainElement.top-constPopupOffset;
		
		var popupBG = this.getElement( this.fPopupBGimg_id );
		popupBG.style.top=this.fMainElement.top-constPopupOffset;
		
		var unavailableLbl = document.getElementById('unavailable-label');
		unavailableLbl.style.top = this.fMainElement.top+this.fMainElement.height + 3;
		
		var switchLanguage = document.getElementById('swap_button');
		switchLanguage.style.top = this.fMainElement.top-constPopupOffset;
				
		this.f_TextBGimg.setOrigin( this.fMainElement.left - 3, this.fMainElement.top-16);
	}
}

var gFromTextBGImage = null;

var gToTextBGImage = null;

var gFromText;
var gToText;

var gBGImage = null;

var gGlobeBodyImage = null;

var gGlobeTextFieldImage = null;

function setupWidget() 
{
    if (window.widget)
    {
        var userLanguages = TranslationEncoder.userLanguagesByPreference();
        var userIdx = 0;
        
        var pref = widget.preferenceForKey("fromLanguage");
        if (pref != null)
        {
            defaultFromLanguage = pref;
        }
        else 
        {
            var foundLanguage = false;
            for ( userIdx = 0; userIdx < userLanguages.length && !foundLanguage; userIdx++ ) {
                for ( var i = 0; i < languageData.length; i++ ) {
                    if ( TranslationEncoder.localizedNameForLanguage(languageData[i].iso) == userLanguages[userIdx] )
                    {
                        foundLanguage = true;
                        break;
                    }
                }
            }
            defaultFromLanguage = i;
            setPreferenceForKey(""+defaultFromLanguage,'fromLanguage');
        }
        
        var pref = widget.preferenceForKey("toLanguage");
        if (pref != null)
        {
            defaultToLanguage = pref;
        }
        else 
        {
            var foundLanguage = false;
            for ( ; userIdx < userLanguages.length && !foundLanguage; userIdx++ ) {
                for ( var j = 0; j < languageData[defaultFromLanguage].array.length; j++ ) {
                    if ( TranslationEncoder.localizedNameForLanguage(languageData[defaultFromLanguage].array[j]) == userLanguages[userIdx] )
                    {
                        foundLanguage = true;
                        break;
                    }
                }
            }
            defaultToLanguage = j;
            setPreferenceForKey(""+defaultToLanguage, 'toLanguage');
        }
    }
    else {
      defaultFromLanguage = 3;
      defaultToLanguage = 3;
    }
    
    // Make sure that our defaultToLanguage index isn't out of bounds
	var supportedLanguages = languageData[defaultFromLanguage].array;
	if ( supportedLanguages[ defaultToLanguage ] == null ) 
	{
		defaultToLanguage = 1;
	}
	
    document.getElementById('Translate_from_label').innerText = getLocalizedString('Translate from');
    document.getElementById('from_text_area').value = "";
    document.getElementById('to_text_area').value = "";
    document.getElementById('unavailable-label').innerText = "";
    document.getElementById('Translate_to_label').innerText = getLocalizedString('To');
    gAllowTextToGrow = true;
    setupLanguageMenusForDefaults();
    
    createElements();
    
    focusFromText();
    
	syncFromLanguageAndFont();
	syncToLanguageAndFont();
	
    // Set up back but do not display
	var doneButton = document.getElementById('done');
	createGenericButton (doneButton, getLocalizedString('Done'), selectDone);
	doneButton.childNodes[1].style.minWidth = '47px';
	var backElement = document.getElementById ('back');
	back.style.display="none";
}

// The constructors for these objects will fail if the page has not loaded yet.
// Defer construction until you are sure the page finished loading.
function createElements()
{
	gFromTextBGImage = new CBGImage( 'from_Text_BGImg_Div', null, null, null, null, 'fromTextCenter', null, null, null, null );

	gToTextBGImage = new CBGImage( 'to_Text_BGImg_Div', null, null, null, null, 'toTextCenter', null, null, null, null );

	gBGImage = new CBGImage( 'front_ImageDiv', 'fg_topl', 'fg_top', 'fg_topr', 'fg_left', 'fg_center', 'fg_right', 'fg_botl', 'fg_bot', 'fg_botr' );

	gGlobeBodyImage = new CBGImage( 'globeBodyDiv', null, null, null, null, 'globeBodyImg', null, null, null, null );

	gGlobeTextFieldImage = new CBGImage( 'globeTextFieldDiv', null, null, null, null, 'globeTextFieldImg', null, null, null, null );

	gFromText = new CTextArea('from_text_area', gFromTextBGImage, 'Translate_from_label', 'from_Language_Static_Label', 'from_Language_PopupMenu', 'from_Language_PopupMenuBGImg', 'greylineHider1');

	gToText = new CTextArea('to_text_area', gToTextBGImage, 'Translate_to_label', 'to_Language_Static_Label', 'to_Language_PopupMenu', 'to_Language_PopupMenuBGImg', 'greylineHider2');

}

// This factory function will build a record of the type used by our classes.
// Instead of hard coding style properties in the Javascript, we will get the
// properties dynamically from the rendering engine.  All initial adjustments to the
// property values should be made in the CSS file.

// For performance reasons, we will still keep our own copy after first getting the values dynamically
// (because we don't want to have to do the text clean up operations performed in this function every time...)
function createElementRecord( id )
{
	if( id )
	{
		var elem = document.getElementById( id );
		
		if( elem )
		{
			// we get the values dynamically for the element (if it exists)
			// these values should have been set in the CSS file (or also the HTML file, but try not to do that...)
			// after we set the values, we need to clean them up to be integers
			
			var imageRecord = { id:id, top: document.defaultView.getComputedStyle(elem, '').getPropertyValue('top'),
									  left: document.defaultView.getComputedStyle(elem, '').getPropertyValue('left'),
									 width: document.defaultView.getComputedStyle(elem, '').getPropertyValue('width'),
									 height: document.defaultView.getComputedStyle(elem, '').getPropertyValue('height') };
			
			// we will treat "auto" as zero.
			// usually, auto means no property was set (or auto was set explicitly)
			if( imageRecord.top == "auto" )
				imageRecord.top = 0;
			if( imageRecord.left == "auto" )
				imageRecord.left = 0;
			
			// when we get a property value, it is returned as a string like "10px"
			// we just want the integer values so we strip all non numeric characters...
			
			var numbersOnlyRegExp = /[0-9]*/i;
			
			if( imageRecord.top != 0 )
			{
				var top = imageRecord.top;
				top = top.match(numbersOnlyRegExp,"");
				imageRecord.top = parseInt( top );
			}
			if( imageRecord.left != 0 )
			{
				var left = imageRecord.left;
				left = left.match(numbersOnlyRegExp,"");
				imageRecord.left = parseInt( left );
			}
			if( imageRecord.width != 0 )
			{
				var width = imageRecord.width;
				width = width.match(numbersOnlyRegExp,"");
				imageRecord.width = parseInt( width );
			}
			if( imageRecord.height != 0 )
			{
				var height = imageRecord.height;
				height = height.match(numbersOnlyRegExp,"");
				imageRecord.height = parseInt( height );
			}
			
			return imageRecord;
		}
		else
			alert( " can't get element with id: " + id );
	}
	return null;
}

// debug output records created by createElementRecord
function dumpElementRecord( rec )
{
	alert( "\r --- element record --- \r " );
	alert( "id: " + rec.id + "\r" );
	alert ("stats:\r" );
	alert( "top: " + rec.top + " left: " + rec.left + " width: " + rec.width + " height: " + rec.height + "\r");
	alert (" --- end --- \r\r");
}

function setupLanguageMenusForDefaults()
{
	populateLanguageSelect(false);					// don't try to match last current language since this is the initial call
	
	if( window.widget ) {
	    document.getElementById('from_Language_Static_Label').innerText = TranslationEncoder.localizedNameForLanguage(languageData[defaultFromLanguage].iso);
	    var supportedLanguages = languageData[defaultFromLanguage].array;
        document.getElementById('to_Language_Static_Label').innerText = TranslationEncoder.localizedNameForLanguage(supportedLanguages[defaultToLanguage]);    
    }
    else 
    {
         document.getElementById('from_Language_Static_Label').innerText = getLocalizedString(languageData[defaultFromLanguage].name);
         document.getElementById('to_Language_Static_Label').innerText = getLocalizedString(languageData[defaultFromLanguage].array[defaultToLanguage]);

    }
     
    document.getElementById ('from_Language_PopupMenu').options[defaultFromLanguage].selected = true;
    document.getElementById ('to_Language_PopupMenu').options[defaultToLanguage].selected = true; 
     
}


function getLocalizedString (key) {
	var returnValue = key;
	try 
	{
		//	This handles the case where localizedStrings[key] doesn't exist
		if ( localizedStrings[key] != null )
			returnValue = localizedStrings[key];
	} 
	catch (ex) 
	{
		//	This handles the case where localizedStrings[] doesn't exist
		debug(key);
	}

	return returnValue;
}

function getTranslationType()
{
    // server likes strings like en_ja to translate from english to japanese, here's where we create the string
    var toLangName = languageData[defaultFromLanguage].array[defaultToLanguage];
    var fromLang = languageData[defaultFromLanguage].code;
    var toLang = "";
    
    for ( i = 0; i < languageData.length; i++ ) {
        if ( languageData[i].iso == toLangName )
            toLang = languageData[i].code;
    }
    
    return fromLang + "_" + toLang;   
}

function getEncodingType(code) {
    if (code == "ja") return "Shift-JIS";
    else if (code == "zh") return "EUC-CN";
    else if (code == "zt") return "BIG5";
    else if (code == "ko") return "EUC-KR";
    else if (code == "ru") return "WINDOWS-1251";
    else if (code == "el") return "ISO-8859-7";
    else 
    return "utf-8";
}

function limit_3 (a, b, c) {
    return a < b ? b : (a > c ? c : a);
}

function computeNextFloat (from, to, ease) {
    return from + (to - from) * ease;
}


function syncFromLanguageAndFont()
{
    fromLanguageFont = fontForLanguage( languageData[defaultFromLanguage].iso  );
    fromLanguageLineHeight = lineHeightForFont( fromLanguageFont );
    document.getElementById('from_text_area').style.fontFamily = fromLanguageFont;
}


function resizeFromTextAsNeeded()
{

	syncFromLanguageAndFont();

    var encodedText = document.getElementById('from_text_area').value;
	var numLines = kMinNumLines;
	var textToTranslate = String( encodedText );
	
	if( TranslationEncoder != null )
		numLines = TranslationEncoder.linesNeededToFitText(textToTranslate, fromLanguageFont, fromLanguageSize, kLineWidth);
	else
		alert("TranslationEncoder not found.  Problem with Translation.widgetplugin?");
	
	if ( numLines > kMaxNumLines )
	{
		textToTranslate = TranslationEncoder.textTrimmedToFitLineCount( textToTranslate, fromLanguageFont, fromLanguageSize, kLineWidth, kMaxNumLines );
		numLines = kMaxNumLines;
		document.getElementById('from_text_area').value = String( textToTranslate );
		gAllowTextToGrow = false;
		doBeep();
	}			
	
	if( numLines != gFromText.fLastLineCount )
		resizeFromText(numLines);
	
	gFromText.fLastTextLen = encodedText.length;
	gFromText.fLastLineCount = numLines;
}

function resizeFromText( lines )
{
	if( lines != parseInt( gFromText.fLastLineCount) )
	{
		var resizeHeight;
		if ( lines < kMinNumLines )
			resizeHeight = kMinTextBoxHeight;
		else
			resizeHeight = (lines * fromLanguageLineHeight);
		resizeHeight = Math.floor( resizeHeight );
		
		if( parseInt( kMinTextBoxHeight ) > parseInt( resizeHeight ) ) // enforce a minimum height
			resizeHeight = kMinTextBoxHeight;
			
		var currentWidgetHeight = gBGImage.getHeight();
		
		if( ( parseInt(currentWidgetHeight) + parseInt(resizeHeight) ) > kAbsoluteMaximumWidgetHeight ) // don't resize the widget too tall
		{
			resizeHeight = (kAbsoluteMaximumWidgetHeight - (parseInt(currentWidgetHeight)));
		}

		var sizeChange = {d_width:0, d_height:parseInt(resizeHeight) - parseInt(gFromText.getHeight())};
		
		if ( Math.abs(sizeChange.d_height) > 0 )
		{
            var animationList = new Array();
            
            animationList.push( createAnimation(gFromText, gFromText.getHeight(), gFromText.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gToText, gToText.getOrigin(), gToText.getOrigin() + sizeChange.d_height, "offset") ); 
            animationList.push( createAnimation(gBGImage, gBGImage.getHeight(), gBGImage.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gGlobeBodyImage, gGlobeBodyImage.getHeight(), gGlobeBodyImage.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gGlobeTextFieldImage, gGlobeTextFieldImage.getHeight(), gGlobeTextFieldImage.getHeight() + sizeChange.d_height, "resize") );            
            runGrowAnimations(animationList, sizeChange, false);
        }
	}
}

var pollTimer = null;
var lastLines = 0;

function resizeToTextAsNeeded()
{
	syncToLanguageAndFont();
	
	var numLines = kMinNumLines;
	var translatedText = String(document.getElementById('to_text_area').value);
	if( TranslationEncoder != null )
		numLines = TranslationEncoder.linesNeededToFitText(translatedText, toLanguageFont, toLanguageSize, kLineWidth);
	else
		alert("TranslationEncoder not found.  Problem with Translation.widgetplugin?");
	if ( numLines > kMaxNumLines )
	{
		translatedText = TranslationEncoder.textTrimmedToFitLineCount( translatedText, toLanguageFont, toLanguageSize, kLineWidth, kMaxNumLines );
		document.getElementById('to_text_area').value = translatedText;
		numLines = kMaxNumLines;
		gAllowTextToGrow = false;
		doBeep();
	}			

	//	N.B., the test to avoid unnecessary resizing and adjust gToText.fLastLineCount
	//		is contained in resizeToText();
	resizeToText( numLines );
}

function resizeToText( lines )
{
    // wait until we are done resizing to resize the translation text
    if ( resizeAnimation.timer != null ) {
        lastLines = lines;
        
        // create a poll timer to poll to see if we are done resizing
        if ( pollTimer == null ) pollTimer = setInterval("resizeToText(lastLines);", kAnimationFrameMilliseconds);
        return;
    }

    if ( lines != parseInt(gToText.fLastLineCount) ) 
    {
        var resizeHeight;
		if ( lines < kMinNumLines )
			resizeHeight = kMinTextBoxHeight;
        else 
        	resizeHeight = (lines * toLanguageLineHeight);
		resizeHeight = Math.floor( resizeHeight );
		
        if ( pollTimer != null ) {
            clearInterval(pollTimer);
            pollTimer = null;
        }
        
        if( kMinTextBoxHeight > resizeHeight ) // enforce a minimum height
            resizeHeight = kMinTextBoxHeight;
    
        var currentWidgetHeight = gBGImage.getHeight();
        
        if( (parseInt(currentWidgetHeight) + parseInt(resizeHeight) > kAbsoluteMaximumWidgetHeight ) )// don't resize the widget too tall
        {
			resizeHeight = (kAbsoluteMaximumWidgetHeight - (parseInt(currentWidgetHeight)));
        }
    
		gToText.fLastLineCount = lines;
		
        var sizeChange = {d_width:0, d_height:parseInt(resizeHeight) - parseInt(gToText.getHeight())};
		if ( Math.abs(sizeChange.d_height) > 0 )
		{
            var animationList = new Array();
            
            animationList.push( createAnimation(gToText, gToText.getHeight(), gToText.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gBGImage, gBGImage.getHeight(), gBGImage.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gGlobeBodyImage, gGlobeBodyImage.getHeight(), gGlobeBodyImage.getHeight() + sizeChange.d_height, "resize") );
            animationList.push( createAnimation(gGlobeTextFieldImage, gGlobeTextFieldImage.getHeight(), gGlobeTextFieldImage.getHeight() + sizeChange.d_height, "resize") );
                    
            runGrowAnimations(animationList, sizeChange, false);
        }
    }
}

function createAnimation( elem, from, to, animType )
{
    var newAnimation = {positionFrom:from, positionTo:to, positionNow:from, element:elem, type:animType, onfinished:null};
    
    return newAnimation;
}

function runGrowAnimations(animationList, sizeChange, doItSlow)
{
	var timeNow = (new Date).getTime();
	var multiplier = (doItSlow ? 10 : 1); // enable slo-mo

	if (resizeAnimation.timer != null) // it is moving... change to new size
	{
		clearInterval(resizeAnimation.timer);
		resizeAnimation.timer = null;
		resizeAnimation.duration -= (timeNow - resizeAnimation.startTime);
		
		for ( var i = 0; i < resizeAnimation.list.length; i++ ) {
            resizeAnimation.list[i].positionFrom = resizeAnimation.list[i].positionNow;
        }
	}
	else
	{
		resizeAnimation.duration = 250 * multiplier;
	}
	
	// if we have a positive growth, resize now so we have room, otherwise 
	// resize at end of animation
	if (window.widget && sizeChange.d_height > 0 )
    {
        window.resizeBy( sizeChange.d_width, sizeChange.d_height);
    }
	
	resizeAnimation.positionTo = parseInt(sizeChange.d_height);
	resizeAnimation.startTime  = timeNow - kAnimationFrameMilliseconds; // set it back one frame.
	resizeAnimation.onfinished = animFinished;
	resizeAnimation.list       = animationList;
	resizeAnimation.timer      = setInterval ("animate();", kAnimationFrameMilliseconds);

	animate();
	
	isCollapsed = false;
}

function animate () {
	var T, animationExtent;
	var time  = (new Date).getTime();
	
	T = limit_3(time-resizeAnimation.startTime, 0, resizeAnimation.duration);
	animationExtent = T / resizeAnimation.duration;

    // maybe convert to integer here for each positionNow variable

	if (T >= resizeAnimation.duration)
	{
        for ( var i = 0; i < resizeAnimation.list.length; i++ ) {
            resizeAnimation.list[i].positionNow = resizeAnimation.list[i].positionTo;
        }
		clearInterval (resizeAnimation.timer);
		resizeAnimation.timer = null;
		
		if ( window.widget && resizeAnimation.positionTo < 0 )
		  window.resizeBy(0, resizeAnimation.positionTo);
	}
	else {
        for ( var i = 0; i < resizeAnimation.list.length; i++ ) {
		  resizeAnimation.list[i].positionNow = computeNextFloat(resizeAnimation.list[i].positionFrom, resizeAnimation.list[i].positionTo, animationExtent);
        }
    }

    // Do the actual update
    for ( var i = 0; i < resizeAnimation.list.length; i++ ) {
        if ( resizeAnimation.list[i].type == "resize" )
            resizeAnimation.list[i].element.resizeTo(0,resizeAnimation.list[i].positionNow);
        else if ( resizeAnimation.list[i].type == "offset" )
            resizeAnimation.list[i].element.offsetTo(resizeAnimation.list[i].element.getLeft(),resizeAnimation.list[i].positionNow);
    }
}

function animFinished() {
	if (window.widget) {
	}
}

var curNumLines = 2;

function translateTokenizeText(text)
{
    var token, tokens, filteredTokens = new Array();
    // Trim the text (note, we use [ \t] here instead of \s
    //		because we want line-breaks to be significant [4029701]
    text = text.replace(/^[ \t]*|[ \t]*$/g,"");
    
    tokens = text.split(/[ \t]/);
    for(var i = 0; i < tokens.length; i++)
        if(tokens[i] != '')
            filteredTokens[filteredTokens.length] = tokens[i];
        
    return filteredTokens;
}

function compareArrays(array1, array2) 
// We don't need a fully generalized array matching routine here because
//	we know we're dealing with two arrays of strings
{
	if (array1.length != array2.length) 
		return false;
	for (var i=0; i<array1.length; i++) 
	{
		if ( array1[ i ] != array2[ i ] )
			return false;
	}
	return true;
}


function translateCut(e,sender)
{
	gAllowTextToGrow = true;
	translateText(); 
}


function translatePaste(e,sender)
{
	translateText();
}

function translateKeyPress(e,sender)
{	
	//	Special-case backspace (U+0008), tab (U+0009), and characters
	//		in the U+F7xx range of the private use area, which
	//		are used by cursor movement and related keys.  We
	//		also pass through anything with the command key pressed.
	//	These never lengthen the text
	//	N.B., if the user happens to use this part of Unicode's private
	//		use area for data interchange -- itself unlikely -- this
	//		code as written will discard it.  However, the data would
	//		be garbage to Systran, anyway, and not valid input for
	//		its translation service, so discarding it doesn't hurt
	if ( e.charCode == 0x0008 || e.charCode == 0x0009 || e.metaKey || 
			(0xF700 <= e.charCode && e.charCode <= 0xF7FF))
		processKeystroke(e,sender);
	else
		checkText(e,sender,String.fromCharCode(e.charCode));
}

function translateKeyUp(e,sender)
{	
	translateText();
}


function processKeystroke(e,sender)
{
	//	Backward and forward delete change the text
	if ( e.charCode == 8 || e.charCode == 63272 ) {
		gAllowTextToGrow = true;
		}
}


function checkText(e,sender,text)
// This function checks to see if the text will put us over our
//	six-line input limit; if so, we throw the event away so that
//	the text area doesn't even get to add it
{
	// If there's no text to process, we don't need to do anything but pass
	//	the event on
	if ( text.length == 0 ) 
		return;

	// If text addition has been blocked, then just kill the event and
	//	abort
	if ( !gAllowTextToGrow ) 
	{
		doBeep();
		e.stopPropagation();
		e.preventDefault();
		return;
	}
	
	// There's text to add and we're not blocking text additions
	//	See if this puts us over the limit
	var encodedText = sender.value + text;
	var numLines = TranslationEncoder.linesNeededToFitText( encodedText, fromLanguageFont, fromLanguageSize, kLineWidth );
	if ( numLines > kMaxNumLines )
	{
		gAllowTextToGrow = ( e.charCode == 10 || e.charCode == 13 );
		doBeep();
		e.stopPropagation();
		e.preventDefault();
	}
	
}

function translateText()
{
    var encodedText = document.getElementById('from_text_area').value;

   // don't do any work if there is no text
    if ( encodedText.length > 0 )
    {
		fetchTranslatedData();
	}
	else 
	{
		var toText = document.getElementById('to_text_area').value;
		if( toText.length > 0 )
			clearTranslationText();	
	}

   	resizeFromTextAsNeeded();
}


function fetchTranslatedDataInternal(parser)
{
 	var encodedText = document.getElementById('from_text_area').value;
 	
 	if( encodedText.length > 0 ) 
 	{
 		if (gLastTranslationXMLRequest != null)
			gLastTranslationXMLRequest.abort();

 		var encodingType = getEncodingType(languageData[defaultFromLanguage].code);
		if (window.widget && encodingType != "utf-8")
		 	encodedText = TranslationEncoder.encodedStringForString(encodedText,encodingType);
		
		gLastTranslationXMLRequest = parser.performXMLRequest(encodedText, encodingType, getTranslationType(), translationLoaded);  	
 	}
}

function fetchTranslatedData()
{
	apple_parser.fetchAndExecute (fetchTranslatedDataInternal);
}


function clearTranslationText()
{
	document.getElementById('unavailable-label').innerText = "";
	document.getElementById('to_text_area').value = "";
    gAllowTextToGrow = true;
	setTimeout("resizeToTextAsNeeded();",0);
}

function translationLoaded(data, errorString)
{

	gLastTranslationXMLRequest = null;
	
	var encodedText = document.getElementById('from_text_area').value;
	syncToLanguageAndFont();
	
	if( encodedText.length > 0 )
	{
		if ( data != null && data.length > 0 )
		{
			if ( !encodedText.match( /\n/ ) ) 
				data = data.replace( /\n/g, ' ' );
			data = data.replace( /\s+$/, '' );
			var toLangName = languageData[defaultFromLanguage].array[defaultToLanguage];
			var numLines = kMinNumLines;
			var translatedText = String( data );
			if( TranslationEncoder != null )
				numLines = TranslationEncoder.linesNeededToFitText(translatedText, toLanguageFont, toLanguageSize, kLineWidth);
			else
				alert("TranslationEncoder not found.  Problem with Translation.widgetplugin?");
			if ( numLines < kMinNumLines )
				numLines = kMinNumLines;
			else if ( numLines > kMaxNumLines )
			{
				translatedText = TranslationEncoder.textTrimmedToFitLineCount( translatedText, toLanguageFont, toLanguageSize, kLineWidth, kMaxNumLines );
				numLines = kMaxNumLines;
				gAllowTextToGrow = false;
			}			
			document.getElementById('to_text_area').value = translatedText;			
			resizeToTextAsNeeded();	
			document.getElementById('unavailable-label').innerText = "";
		}
		else
		{
			clearTranslationText();
			document.getElementById('unavailable-label').innerText = getLocalizedString('Data unavailable.');
		}
    }
    else
    	clearTranslationText();
}

function toTextChanged()
{
	resizeToTextAsNeeded();
}

if (window.widget) {
	widget.onhide = onhide;
	widget.onshow = onshow;
}

var flipShown = false;

function populateLanguageSelect(enforceCurrLanguage)
{
    // populate the fromLanguage select
	var select = document.getElementById('from_Language_PopupMenu');
    var c = languageData.length;

    for ( var i = 0; i < c; ++i )
    {
        var element = document.createElement("option");
        if ( window.widget )
            element.innerText = TranslationEncoder.localizedNameForLanguage(languageData[i].iso);
        else
            element.innerText = getLocalizedString(languageData[i].name);
        select.appendChild (element);
    }
    
    populateToLanguageSelect(enforceCurrLanguage);
}

function populateToLanguageSelect(enforceCurrLanguage)
{    
    // get the last toLanguage
   	var currentToLanguage = document.getElementById('to_Language_Static_Label').innerText;
    
    // remove all children

    var select = document.getElementById('to_Language_PopupMenu');
	while (select.hasChildNodes()) 
		select.removeChild(select.firstChild);
   
	
	// translate the languages for the popup and add them
	var supportedLanguages = languageData[defaultFromLanguage].array;
	var languageLength = supportedLanguages.length;
	var languageStrings = new Array;
	
	if( window.widget ) 
	{
		for( var i = 0; i < languageLength; i++) 
			languageStrings[i] = TranslationEncoder.localizedNameForLanguage(supportedLanguages[i]);
	} 
	else 
	{
		for( var i = 0; i < languageLength; i++) 
			languageStrings[i] = getLocalizedString(supportedLanguages[i]);
	}
	
	for( var i = 0; i < languageLength; i++ )
	{
		var element = document.createElement("option");
		element.innerText = languageStrings[i];
		select.appendChild(element);
	}

	// try to find the old toLanguage to keep things consistent for the user
	// so, when the fromLanguage is changed, the same toLanguage (if available) is also set
	if( enforceCurrLanguage == true ) 
	{
		var defaultToLanguage = 0;
		if ( currentToLanguage.length > 0 ) 
		{
			for ( i = 0; i < languageLength; i++ ) 
			{
			   if ( languageStrings[i] == currentToLanguage ) {
				   defaultToLanguage = i;
				   break;
			   }
			}
		}		
		select.options[defaultToLanguage].selected = true;
    }
}

function fromLanguageChanged (select,smartSelectToLang)
{
    var translateElem = document.getElementById('from_text_area');
    var translationElem = document.getElementById('to_text_area');

    // set the UI
	document.getElementById('unavailable-label').innerText = "";
    defaultFromLanguage = select.selectedIndex;
    if ( window.widget )
        document.getElementById('from_Language_Static_Label').innerText = TranslationEncoder.localizedNameForLanguage(languageData[defaultFromLanguage].iso);
    else
        document.getElementById('from_Language_Static_Label').innerText = getLocalizedString(languageData[defaultFromLanguage].name);
 
 	focusFromText();
 	clearTranslationText();
    resizeFromTextAsNeeded();
            
    // with this fromLanguage, repopulate the toLanguage select, try to find the last toLanguage
    populateToLanguageSelect(smartSelectToLang);
    // perform the new translation given the new from/to languages
    toLanguageChanged(document.getElementById('to_Language_PopupMenu'));
    
    syncFromLanguageAndFont();
    
}

function handleFromLanguageChanged(select)
{
	// blank the text area, to prevent mismatch between language of content, and language selected in menu
	document.getElementById('from_text_area').value = "";
    gAllowTextToGrow = true;
	
	fromLanguageChanged(select,true); // destination text area will be blanked in here
    
    // save to preferences
    if (window.widget) 
    {
        widget.setPreferenceForKey (defaultFromLanguage.toString(), "fromLanguage");
        widget.setPreferenceForKey (defaultToLanguage.toString(), "toLanguage");
    }
}


function syncToLanguageAndFont()
{
    var toLangName = languageData[defaultFromLanguage].array[defaultToLanguage];
    toLanguageFont = fontForLanguage( toLangName );
    toLanguageLineHeight = lineHeightForFont( toLanguageFont );
    document.getElementById('to_text_area').style.fontFamily = toLanguageFont;
}


function toLanguageChanged (select)
{
    defaultToLanguage = select.selectedIndex;
    
    // set the UI
    var toLangName = languageData[defaultFromLanguage].array[defaultToLanguage];
    if ( window.widget )
        document.getElementById('to_Language_Static_Label').innerText = TranslationEncoder.localizedNameForLanguage(toLangName);
    else
        document.getElementById('to_Language_Static_Label').innerText = getLocalizedString(toLangName);
	focusFromText();
	
	// Retranslate
	clearTranslationText();
    translateText();

    // reset the UI
    syncToLanguageAndFont();
    
}

function handleToLanguageChanged(select)
{
	toLanguageChanged(select);
    
    // set the prefs to the most recent toLanguage
    if (window.widget) 
    	widget.setPreferenceForKey (defaultToLanguage.toString(), "toLanguage");
}

function switchLanguages()
{
	if( xmlRequestAwaitingReply() )
		return;
		
	var fromIndex = -1, toIndex = -1;

	var fromSelect = document.getElementById('from_Language_PopupMenu');
	var fromSelectedIndex = fromSelect.selectedIndex;
	var fromLanguageISO = languageData[fromSelectedIndex].iso;
	
	var toSelect = document.getElementById('to_Language_PopupMenu');
	var toSelectedIndex = toSelect.selectedIndex;
	var toLanguageISO = languageData[fromSelectedIndex].array[toSelectedIndex];

    var i;
    
    for ( i = 0; i < languageData.length; i++ ) {
        if ( toLanguageISO == languageData[i].iso ) {
            fromIndex = i;
            break;
        }
    }
    for ( i = 0; i < languageData[fromIndex].array.length; i++ ) {
        if ( fromLanguageISO == languageData[fromIndex].array[i] ) {
            toIndex = i;
            break;
        }
    }
    if ( fromIndex != -1 && toIndex != -1 ) {
        var translationText = document.getElementById('to_text_area').value;
        
        document.getElementById('to_text_area').value = document.getElementById('from_text_area').value;
        document.getElementById('from_text_area').value = translationText;
        
        defaultFromLanguage = toIndex;
        
        if ( window.widget ) {
            fromLanguageName = TranslationEncoder.localizedNameForLanguage(toLanguageISO);
            toLanguageName = TranslationEncoder.localizedNameForLanguage(fromLanguageISO);
        }
        var fromSelect = document.getElementById ('from_Language_PopupMenu');
        var toSelect = document.getElementById ('to_Language_PopupMenu');
        
        document.getElementById('to_Language_Static_Label').innerText = fromLanguageName;
        document.getElementById('from_Language_Static_Label').innerText = toLanguageName;
        
        fromSelect.options[fromIndex].selected = true;
		fromLanguageChanged(fromSelect,false);
        defaultToLanguage = fromIndex;
        toSelect.options[toIndex].selected = true;
		toLanguageChanged(toSelect);
	    translateText();
    }    
}

function xmlRequestAwaitingReply()
{
	return( gLastTranslationXMLRequest != null );
}


function onshow() {

	resizeFromTextAsNeeded();
	resizeToTextAsNeeded();
    document.getElementById ('from_text_area').focus();
    document.getElementById ('to_text_area').blur();
}

function onhide() {
    document.getElementById ('from_text_area').blur();
    document.getElementById ('to_text_area').blur();
}


if (window.widget)
{
	widget.onshow = onshow;
	widget.onhide = onhide;
}

function clickOnProvider(event)
{
    widget.openURL("http://www.systransoft.com");
}


function enterflip(event)
{
    document.getElementById('flip_rollie').style.display = 'block';
}

function exitflip(event)
{
    document.getElementById('flip_rollie').style.display = 'none';
}

var flipAnimation = {duration:0, starttime:0, to:1.0, now:0.0, from:0.0, element:null, timer:null};
var flipShown = false;

function flipAnimate()
{
	var T;
	var ease;
	var time = (new Date).getTime();
		
	
	T = limit_3(time-flipAnimation.starttime, 0, flipAnimation.duration);
	
	if (T >= flipAnimation.duration)
	{
		clearInterval (flipAnimation.timer);
		flipAnimation.timer = null;
		flipAnimation.now = flipAnimation.to;
	}
	else
	{
		ease = 0.5 - (0.5 * Math.cos(Math.PI * T / flipAnimation.duration));
		flipAnimation.now = computeNextFloat (flipAnimation.from, flipAnimation.to, ease);
	}
	
	flipAnimation.element.style.opacity = flipAnimation.now;
}

function mousemove (event)
{
	if (!flipShown)
	{
		// fade in the flip widget
		if (flipAnimation.timer != null)
		{
			clearInterval (flipAnimation.timer);
			flipAnimation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - kAnimationFrameMilliseconds; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById ('flip');
		flipAnimation.timer = setInterval ("flipAnimate();", kAnimationFrameMilliseconds);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 1.0;
		flipAnimate();
		flipShown = true;
	}
}

function mouseexit (event)
{
	if (flipShown)
	{
		// fade in the flip widget
		if (flipAnimation.timer != null)
		{
			clearInterval (flipAnimation.timer);
			flipAnimation.timer  = null;
		}
		
		var starttime = (new Date).getTime() - kAnimationFrameMilliseconds; // set it back one frame
		
		flipAnimation.duration = 500;
		flipAnimation.starttime = starttime;
		flipAnimation.element = document.getElementById ('flip');
		flipAnimation.timer = setInterval ("flipAnimate();", kAnimationFrameMilliseconds);
		flipAnimation.from = flipAnimation.now;
		flipAnimation.to = 0.0;
		flipAnimate();
		flipShown = false;
	}
	
}


function showbackside(event) 
{
	if( flipShown )
	{
		var front = document.getElementById("front");
		var frontimage = document.getElementById("front_ImageDiv");
		var back = document.getElementById("back");

		if (window.widget)
			widget.prepareForTransition("ToBack");
	
		front.style.display="none";
		frontimage.style.display="none";
		back.style.display="block";
	
		if (window.widget)
			setTimeout('widget.performTransition();', 0);
				
		document.getElementById('flip_rollie').style.display = "none";
		flipShown = false;
	}
}


function swapMouseDown(event)
{
    document.getElementById('swap_button').src = 'Images/translator_swap_ON.png';
}

function swapMouseUp(event)
{
    document.getElementById('swap_button').src = 'Images/translator_swap_OFF.png';
    switchLanguages();
}

function selectDone ()
{
	var front = document.getElementById("front");
	var frontimage = document.getElementById("front_ImageDiv");
	var back = document.getElementById("back");

	if (window.widget)
		widget.prepareForTransition("ToFront");
	
	front.style.display="block";
	frontimage.style.display="block";
	back.style.display="none";
	
	if (window.widget)
		setTimeout('widget.performTransition();', 0);
				
	onshow();
}

function focusFromText()
{
	gFromText.focus( true );
	gToText.focus( false );
}


function focusToText()
{
	gToText.focus( true );
	gFromText.focus( false );
}

function blurText()
{
	gToText.focus( false );
	gFromText.focus( false );
}


function clearLastXMLRequest()
{
	gLastTranslationXMLRequest = null;
}


function isXMLRequestLastOneSent(xml_request)
{
    return( xml_request == gLastTranslationXMLRequest);
}



function debug(msg) {
	if (!debug.box) {
		debug.box = document.createElement("div");
		debug.box.setAttribute("style", "background-color: white; " +
										"font-family: monospace; " +
										"border: solid black 3px; " +
										"position: absolute;top:300px;" +
										"padding: 10px;");
		document.body.appendChild(debug.box);
		debug.box.innerHTML = "<h1 style='text-align:center'>Debug Output</h1>";
	}
	
	var p = document.createElement("p");
	p.appendChild(document.createTextNode(msg));
	debug.box.appendChild(p);
}

